@@ -1,7 +1,7 @@
 /*
  * gimmix-covers.c
  *
- * Copyright (C) 2008 Priyank Gosalia
+ * Copyright (C) 2008-2009 Priyank Gosalia
  *
  * Gimmix is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -36,6 +36,7 @@
 #include "wejpconfig.h"
 #include "gimmix-metadata.h"
 #include "gimmix-covers.h"
+#include "gimmix-tagedit.h"
 
 #define DEFAULT_COVER	"gimmix-album.png"
 #define COVERS_DIR	".gimmix/covers"
@@ -61,12 +62,14 @@ extern ConfigFile	conf;
 extern MpdObj		*gmo;
 extern mpd_Song		*glob_song_info;
 extern GimmixTooltip 	*tooltip;
+extern GtkWidget	*main_window;
 
 static ConfigFile	cover_db;
 static CURL		*curl;
 static char		*cover_image_path;
 static GtkWidget	*gimmix_metadata_image;
 static GtkWidget	*gimmix_plcbox_image;
+static GtkWidget	*gimmix_plcbox_eventbox;
 
 GtkWidget		*gimmix_plcbox_frame;
 static GMutex		*mutex = NULL;
@@ -77,15 +80,13 @@ static CoverNode* gimmix_covers_plugin_get_metadata (char *arg1, char *arg1d, ch
 /* Get the fallback cover image of specified size */
 static GdkPixbuf* gimmix_covers_plugin_get_default_cover (guint width, guint height);
 
-/* Set the cover image for metadata section */
-static void gimmix_covers_plugin_set_metadata_image (GdkPixbuf *pixbuf);
-
 static gboolean gimmix_covers_plugin_download (const char *url, const char *file);
 static CoverNode* gimmix_cover_node_new (void);
 static gchar *gimmix_url_encode (const char *string);
 static void gimmix_covers_plugin_cover_db_init (void);
 static void gimmix_covers_plugin_cover_db_save (void);
 static void gimmix_covers_plugin_find_cover (mpd_Song *s);
+static void gimmix_cover_plugin_save_cover (char *artist, char *album);
 
 static void
 cb_gimmix_covers_plugin_plcbox_size_allocated (GtkWidget *widget, GtkAllocation *a, gpointer data)
@@ -98,6 +99,159 @@ cb_gimmix_covers_plugin_plcbox_size_allocated (GtkWidget *widget, GtkAllocation
 	return;
 }
 
+static void
+cb_gimmix_covers_plugin_cover_file_preview (GtkFileChooser *file_chooser, gpointer data)
+{
+	GtkWidget	*preview = NULL;
+	char		*filename = NULL;
+	GdkPixbuf	*pixbuf = NULL;
+	gboolean	have_preview;
+	
+	preview = GTK_WIDGET (data);
+	filename = gtk_file_chooser_get_preview_filename (file_chooser);
+	if (filename == NULL)
+		return;
+	pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 128, 128, NULL);
+	have_preview = (pixbuf != NULL);
+	g_free (filename);
+	if (pixbuf != NULL)
+	{
+		gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
+		g_object_unref (pixbuf);
+	}
+	gtk_file_chooser_set_preview_widget_active (file_chooser, have_preview);
+
+	return;
+}
+
+static void
+cb_gimmix_covers_plugin_refetch_cover (void)
+{
+	g_thread_create ((GThreadFunc)gimmix_covers_plugin_update_cover,
+				FALSE,
+				FALSE,
+				NULL);
+
+	return;
+}
+
+static void
+cb_gimmix_covers_plugin_set_cover_from_file (void)
+{
+	GtkWidget	*dialog;
+	GtkFileFilter	*filter = NULL;
+	GtkImage	*preview;
+	mpd_Song	*song = NULL;
+	gchar		*artist = NULL;
+	gchar		*album = NULL;
+	
+	if (!(song=mpd_playlist_get_current_song(gmo)))
+		return;
+	
+	artist = (song->artist != NULL) ? g_strdup (song->artist) : NULL;
+	album = (song->album != NULL) ? g_strdup (song->album) : NULL;
+	dialog = gtk_file_chooser_dialog_new ("Open File",
+					GTK_WINDOW(main_window),
+					GTK_FILE_CHOOSER_ACTION_OPEN,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					NULL);
+	preview = (GtkImage*)gtk_image_new ();
+	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER(dialog), GTK_WIDGET(preview));
+	g_signal_connect (GTK_FILE_CHOOSER(dialog), "update-preview", G_CALLBACK (cb_gimmix_covers_plugin_cover_file_preview), preview);
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, "Images (.jpg)");
+	gtk_file_filter_add_pattern (filter, "*.jpg");
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), filter);
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+	{
+		char	*filename = NULL;
+		gchar	*contents = NULL;
+		gsize	len = 0;
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		if (g_file_get_contents(filename, &contents, &len, NULL))
+		{
+			gchar	*temp = NULL;
+			char	*p = cfg_get_path_to_config_file (COVERS_DIR);
+			temp = g_strdup_printf ("%s/temp.jpg", p);
+			g_free (p);
+			if (g_file_set_contents (temp, contents, len, NULL))
+			{
+				if (artist!=NULL && album!=NULL)
+				{
+					gimmix_cover_plugin_save_cover (artist, album);
+					g_thread_create ((GThreadFunc)gimmix_covers_plugin_update_cover,
+							FALSE,
+							FALSE,
+							NULL);
+					g_free (artist);
+					g_free (album);
+				}
+			}
+			else
+			{
+				gimmix_error ("There was an error while setting the album cover. Please try using a different image.");
+			}
+			g_free (temp);
+		}
+		g_free (filename);
+	}
+	gtk_widget_destroy (dialog);
+	
+	return;
+}
+
+static void
+cb_gimmix_covers_plugin_plc_popup (GtkWidget *widget, GdkEventButton *event, gpointer data)
+{
+	GtkWidget 	*menu = NULL;
+	GtkWidget 	*menu_item = NULL;
+	GtkWidget	*image = NULL;
+
+	if (event->button == 3) /* If right click */
+	{
+		menu = gtk_menu_new ();
+
+		image = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_MENU);
+		menu_item = gtk_image_menu_item_new_with_label (_("Set cover from file"));
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menu_item), image);
+		g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (cb_gimmix_covers_plugin_set_cover_from_file), NULL);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+		gtk_widget_show (menu_item);
+		
+		image = gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_MENU);
+		menu_item = gtk_image_menu_item_new_with_label (_("Re-fetch cover"));
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menu_item), image);
+		g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK(cb_gimmix_covers_plugin_refetch_cover), NULL);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+		gtk_widget_show (menu_item);
+		
+		gtk_widget_show (menu);
+		gtk_menu_popup (GTK_MENU(menu),
+					NULL,
+					NULL,
+					NULL,
+					NULL,
+					3,
+					gtk_get_current_event_time());
+	}
+}
+
+static void
+gimmix_covers_plugin_proxy_init (nxml_t *n)
+{
+	char	*proxy = NULL;
+	
+	if (!strncasecmp(cfg_get_key_value(conf,"proxy_enable"),"true",4))
+	{
+		proxy = gimmix_config_get_proxy_string ();
+		nxml_set_proxy (n, proxy, NULL);
+		g_free (proxy);
+	}
+	
+	return;
+}
+
 void
 gimmix_covers_plugin_init (void)
 {
@@ -116,6 +270,7 @@ gimmix_covers_plugin_init (void)
 		FILE *fp = fopen (cpath, "w");
 		fclose (fp);
 	}
+	g_free (cpath);
 	
 	/* initialize curl */
 	curl = curl_easy_init ();
@@ -127,11 +282,14 @@ gimmix_covers_plugin_init (void)
 	gimmix_covers_plugin_cover_db_init ();
 	
 	/* initialize metadata widgets */
+	gimmix_plcbox_eventbox = glade_xml_get_widget (xml, "cover_event_box");
 	gimmix_plcbox_image = glade_xml_get_widget (xml, "gimmix_plcbox_image");
 	gimmix_metadata_image = glade_xml_get_widget (xml, "gimmix_metadata_image");
 	gimmix_plcbox_frame = glade_xml_get_widget (xml, "gimmix_plc_image_frame");
 	
 	/* some signals */
+	g_signal_connect (gimmix_plcbox_eventbox, "button_press_event", G_CALLBACK(cb_gimmix_covers_plugin_plc_popup), NULL);
+	
 	/* an ugly way to calculate size of the album picture placeholder */
 	widget = glade_xml_get_widget (xml,"plcvbox");
 	g_signal_connect (widget, "size-allocate", G_CALLBACK(cb_gimmix_covers_plugin_plcbox_size_allocated), NULL);
@@ -147,6 +305,10 @@ void
 gimmix_covers_plugin_cleanup (void)
 {
 	curl_easy_cleanup (curl);
+	if (mutex)
+	{
+		g_mutex_free (mutex);
+	}
 	
 	return;
 }
@@ -236,10 +398,18 @@ gimmix_covers_plugin_download (const char *url, const char *file)
 		{
 			FILE *outfile = NULL;
 			char *path = NULL;
+			char *proxy = NULL;
 
 			//path = g_strdup_printf ("%s/%s", cfg_get_path_to_config_file(COVERS_DIR), file);
 			//g_print (path);
 			outfile = fopen (file, "w");
+			/* use a proxy if enabled */
+			if (gimmix_config_get_bool("proxy_enable"))
+			{
+				proxy = gimmix_config_get_proxy_string ();
+				curl_easy_setopt (curl, CURLOPT_PROXY, proxy);
+				g_free (proxy);
+			}
 			curl_easy_setopt (curl, CURLOPT_URL, url);
 			curl_easy_setopt (curl, CURLOPT_WRITEDATA, outfile);
 			curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, __curl_write_func);
@@ -274,7 +444,7 @@ gimmix_covers_plugin_get_metadata (char *arg1, char *arg1d, char *arg2, char *ar
 	
 	u_artist = gimmix_url_encode (arg1d);
 	u_title = gimmix_url_encode (arg2d);
-	location = cfg_get_key_value (conf,"coverart_location");
+	location = cfg_get_key_value (conf, "coverart_location");
 	if (!arg1 && !arg1d)
 	{
 		url = g_strdup_printf (AMAZON_URL1, location, AMAZON_KEY, arg2, u_title);
@@ -286,6 +456,8 @@ gimmix_covers_plugin_get_metadata (char *arg1, char *arg1d, char *arg2, char *ar
 	//g_print ("%s\n", url);
 
 	e = nxml_new (&nxml);
+	nxml_set_timeout (nxml, 20);
+	gimmix_covers_plugin_proxy_init (nxml);
 	nxml_parse_url (nxml, url);
 	nxml_root_element (nxml, &nroot);
 	nxml_find_element (nxml, nroot, "Items", &ndata);
@@ -357,6 +529,7 @@ gimmix_cover_plugin_save_cover (char *artist, char *album)
 	char	*old_path = NULL;
 	char	*new_path = NULL;
 	char	*key = NULL;
+	char	*temp = NULL;
 	
 	if (artist == NULL || album == NULL)
 		return;
@@ -364,9 +537,11 @@ gimmix_cover_plugin_save_cover (char *artist, char *album)
 	album_e = gimmix_url_encode (album);
 	
 	/* save cover art */
-	old_path = g_strdup_printf ("%s/temp.jpg", cfg_get_path_to_config_file(COVERS_DIR));
-	new_path = g_strdup_printf ("%s/%s-%s.jpg", cfg_get_path_to_config_file(COVERS_DIR), artist_e, album_e);
+	temp = cfg_get_path_to_config_file (COVERS_DIR);
+	old_path = g_strdup_printf ("%s/temp.jpg", temp);
+	new_path = g_strdup_printf ("%s/%s-%s.jpg", temp, artist_e, album_e);
 	g_rename (old_path, new_path);
+	g_free (temp);
 	
 	/* okay, add an entry to covers.db */
 	key = g_strdup_printf ("%s-%s", artist, album);
@@ -391,6 +566,7 @@ gimmix_covers_plugin_save_albuminfo (char *artist, char *album, char *info)
 	char	*path = NULL;
 	char	*artist_e = NULL;
 	char	*album_e = NULL;
+	char	*temp = NULL;
 	
 	if (info == NULL || !strlen(info))
 		return;
@@ -398,10 +574,14 @@ gimmix_covers_plugin_save_albuminfo (char *artist, char *album, char *info)
 	/* save album info */
 	artist_e = gimmix_url_encode (artist);
 	album_e = gimmix_url_encode (album);
-	path = g_strdup_printf ("%s/%s-%s.albuminfo", cfg_get_path_to_config_file(COVERS_DIR), artist_e, album_e);
-	fp = fopen (path, "w");
-	fprintf (fp, info);
-	fclose (fp);
+	temp = cfg_get_path_to_config_file (COVERS_DIR);
+	path = g_strdup_printf ("%s/%s-%s.albuminfo", temp, artist_e, album_e);
+	if ((fp=fopen(path, "w")))
+	{
+		fprintf (fp, info);
+		fclose (fp);
+	}
+	g_free (temp);
 	g_free (path);
 	g_free (artist_e);
 	g_free (album_e);
@@ -418,6 +598,8 @@ gimmix_covers_plugin_get_albuminfo (mpd_Song *s)
 	char	*path = NULL;
 	char	line[256] = "";
 	char	*ret = NULL;
+	char	*temp = NULL;
+	char	*p = NULL;
 	
 	if (s == NULL)
 	{
@@ -429,7 +611,8 @@ gimmix_covers_plugin_get_albuminfo (mpd_Song *s)
 	}
 	artist_e = gimmix_url_encode (s->artist);
 	album_e = gimmix_url_encode (s->album);
-	path = g_strdup_printf ("%s/%s-%s.albuminfo", cfg_get_path_to_config_file(COVERS_DIR), artist_e, album_e);
+	temp = cfg_get_path_to_config_file (COVERS_DIR);
+	path = g_strdup_printf ("%s/%s-%s.albuminfo", temp, artist_e, album_e);
 	g_free (artist_e);
 	g_free (album_e);
 	
@@ -440,13 +623,21 @@ gimmix_covers_plugin_get_albuminfo (mpd_Song *s)
 		{
 			str = g_string_append (str, line);
 		}
+		fclose (fp);
 		ret = g_strdup (str->str);
+		g_free (path);
+		g_free (temp);
+		g_string_free (str, TRUE);
 		return ret;
 	}
 	
 	artist_e = gimmix_url_encode (s->performer);
 	album_e = gimmix_url_encode (s->album);
-	path = g_strdup_printf ("%s/%s-%s.albuminfo", cfg_get_path_to_config_file(COVERS_DIR), artist_e, album_e);
+	g_free (path);
+	p = cfg_get_path_to_config_file (COVERS_DIR);
+	path = g_strdup_printf ("%s/%s-%s.albuminfo", p, artist_e, album_e);
+	g_free (p);
+	g_free (temp);
 	g_free (artist_e);
 	g_free (album_e);
 	
@@ -457,9 +648,13 @@ gimmix_covers_plugin_get_albuminfo (mpd_Song *s)
 		{
 			str = g_string_append (str, line);
 		}
+		fclose (fp);
 		ret = g_strdup (str->str);
+		g_free (path);
+		g_string_free (str, TRUE);
 		return ret;
 	}
+	g_free (path);
 	
 	return ret;
 }
@@ -507,13 +702,7 @@ gimmix_covers_plugin_get_cover_image_of_size (guint width, guint height)
 	}
 	else
 	{
-		mpd_Song *s = NULL;
-		sleep (2);
-		s = mpd_playlist_get_current_song (gmo);
-		
-		gimmix_covers_plugin_find_cover (s);
-		
-		if (s == NULL || cover_image_path == NULL)
+		if (!cover_image_path)
 		{
 			/* set default image */
 			//g_print ("cover_image_path is NULL\n");
@@ -540,6 +729,7 @@ gimmix_covers_plugin_find_cover (mpd_Song *s)
 	if (s != NULL)
 	{
 		char *result = NULL;
+		guint len = 0;
 		
 		/* first look into the local cover database */
 		if (!s->artist || !s->album)
@@ -547,7 +737,9 @@ gimmix_covers_plugin_find_cover (mpd_Song *s)
 			gimmix_covers_plugin_set_cover_image_path (result);
 			return;
 		}
-		strncpy (sartist, s->artist, strlen(s->artist));
+		if (s->artist)
+			len = strlen (s->artist);
+		strncpy (sartist, s->artist, len);
 		g_strstrip (sartist);
 		strncpy (salbum, s->album, strlen(s->album));
 		g_strstrip (salbum);
@@ -559,17 +751,21 @@ gimmix_covers_plugin_find_cover (mpd_Song *s)
 		temp = g_strdup_printf ("%s-%s", sartist, salbum);
 		gimmix_strcrep (temp, ' ', '_');
 		result = cfg_get_key_value (cover_db, temp);
+		g_free (temp);
+		//g_print ("result: %s\n", result);
 		if (result!=NULL)
 		{
 			gimmix_covers_plugin_set_cover_image_path (result);
-			//g_print ("found on localdisk\n");
+			g_print ("cover found on disk\n");
 			return;
 		}
 		/* if not found locally, fetch it from amazon */
 		else
 		{
 			//g_print ("beginning to fetch \n");
-			temp = g_strdup_printf ("%s/temp.jpg", cfg_get_path_to_config_file(COVERS_DIR));
+			char *ptr = cfg_get_path_to_config_file (COVERS_DIR);
+			temp = g_strdup_printf ("%s/temp.jpg", ptr);
+			g_free (ptr);
 			node = gimmix_covers_plugin_get_metadata ("Artist", sartist, "Title", salbum);
 			if (node!=NULL)
 			{
@@ -582,6 +778,7 @@ gimmix_covers_plugin_find_cover (mpd_Song *s)
 					gimmix_covers_plugin_find_cover (s);
 				}
 				g_free (node);
+				g_free (temp);
 				return;
 			}
 			node = gimmix_covers_plugin_get_metadata (NULL, NULL, "Title", salbum);
@@ -596,6 +793,7 @@ gimmix_covers_plugin_find_cover (mpd_Song *s)
 					gimmix_covers_plugin_find_cover (s);
 				}
 				g_free (node);
+				g_free (temp);
 				return;
 			}
 			else
@@ -612,6 +810,7 @@ gimmix_covers_plugin_find_cover (mpd_Song *s)
 						gimmix_covers_plugin_find_cover (s);
 					}
 					g_free (node);
+					g_free (temp);
 					return;
 				}
 				else
@@ -627,84 +826,55 @@ gimmix_covers_plugin_find_cover (mpd_Song *s)
 	return;
 }
 
-static void
-gimmix_covers_plugin_set_metadata_image (GdkPixbuf *pixbuf)
-{
-	gtk_image_set_from_pixbuf (GTK_IMAGE(gimmix_metadata_image), pixbuf);
-	
-	return;
-}
-
 /* if default = TRUE, set the default cover */
 void
 gimmix_covers_plugin_update_cover (gboolean defaultc)
 {
 	guint		height;
 	GdkPixbuf	*pixbuf = NULL;
-	mpd_Song	*s = NULL;
 
 	g_mutex_lock (mutex);
 	height = h3_size;
-	if (defaultc)
+	if (!defaultc)
 	{
-		pixbuf = gimmix_covers_plugin_get_default_cover (96, height);
-		gtk_image_set_from_pixbuf (GTK_IMAGE(gimmix_plcbox_image), pixbuf);
-		g_object_unref (pixbuf);
-		pixbuf = gimmix_covers_plugin_get_default_cover (64, 64);
-		gimmix_covers_plugin_set_metadata_image (pixbuf);
-		g_object_unref (pixbuf);
-		if (!strncasecmp(cfg_get_key_value(conf,"enable_systray"),"true",4))
-		{
-			if (!strncasecmp(cfg_get_key_value(conf,"enable_notification"),"true",4))
-			{
-				pixbuf = gimmix_covers_plugin_get_default_cover (48, 48);
-				gimmix_tooltip_set_icon (tooltip, pixbuf);
-				g_object_unref (pixbuf);
-			}
-		}
-		g_mutex_unlock (mutex);
-		return;
+		mpd_Song *s = NULL;
+		mpd_Song *sng = mpd_newSong ();
+		while (!(s=mpd_playlist_get_current_song(gmo)))
+			sleep (1);
+		sng = mpd_songDup (s);
+		gimmix_covers_plugin_find_cover (sng);
+		pixbuf = gimmix_covers_plugin_get_cover_image_of_size (96, height);
+		mpd_freeSong (sng);
 	}
 	else
 	{
-		pixbuf = gimmix_covers_plugin_get_cover_image_of_size (96, height);
+		pixbuf = gimmix_covers_plugin_get_default_cover (96, height);
 	}
 	
-	sleep (2);
-	if (mpd_player_get_state(gmo)!=MPD_PLAYER_STOP)
-	{
-		if (mpd_playlist_get_playlist_length(gmo))
-			s = mpd_playlist_get_current_song (gmo);
-		else
-			s = NULL;
-	}
-
 	if (pixbuf != NULL)
 	{
-		char *areview = NULL;
-		
 		/* main window cover art */
+		gdk_threads_enter ();
 		gtk_image_set_from_pixbuf (GTK_IMAGE(gimmix_plcbox_image), pixbuf);
-		g_object_unref (pixbuf);
-		
-		/* metadata cover art */
-		if (s!=NULL)
-			pixbuf = gimmix_covers_plugin_get_cover_image_of_size (64, 64);
-		else
-			pixbuf = gimmix_covers_plugin_get_default_cover (64, 64);
-		gimmix_covers_plugin_set_metadata_image (pixbuf);
+		gdk_flush ();
+		gdk_threads_leave ();
 		g_object_unref (pixbuf);
 		
 		/* metadata albuminfo */
-		areview = gimmix_covers_plugin_get_albuminfo (s);
-		gimmix_metadata_set_song_details (s, areview);
-		if (areview)
-			g_free (areview);
-		
+		// gimmix_metadata_set_song_details (s, areview);
+
+		/* tag editor cover image */
+		pixbuf = gimmix_covers_plugin_get_cover_image_of_size (96, 96);
+		gdk_threads_enter ();
+		gimmix_tag_editor_set_cover_image (pixbuf);
+		gdk_flush ();
+		gdk_threads_leave ();
+		g_object_unref (pixbuf);
+
 		/* also system tray tooltip image */
-		if (!strncasecmp(cfg_get_key_value(conf,"enable_systray"),"true",4))
+		if (gimmix_config_get_bool("enable_systray"))
 		{
-			if (!strncasecmp(cfg_get_key_value(conf,"enable_notification"),"true",4))
+			if (gimmix_config_get_bool("enable_notification"))
 			{
 				pixbuf = gimmix_covers_plugin_get_cover_image_of_size (48, 48);
 				gimmix_tooltip_set_icon (tooltip, pixbuf);
@@ -713,7 +883,7 @@ gimmix_covers_plugin_update_cover (gboolean defaultc)
 		}
 	}
 	g_mutex_unlock (mutex);
-	
+
 	return;
 }
 
