@@ -1,7 +1,7 @@
 /*
  * gimmix-playlist.c
  *
- * Copyright (C) 2006-2008 Priyank Gosalia
+ * Copyright (C) 2006-2009 Priyank Gosalia
  *
  * Gimmix is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -26,8 +26,16 @@
 #include "gimmix-playlist.h"
 #include "gimmix-tagedit.h"
 
-#define GIMMIX_MEDIA_ICON 		"gimmix_logo_small.png"
+#define GIMMIX_MEDIA_ICON 	"gimmix_logo_small.png"
 #define GIMMIX_PLAYLIST_ICON 	"gimmix_playlist.png"
+#define LIBMPD_VER		(LIBMPD_VERSION * 10000)
+
+typedef enum {
+	COLUMN_TITLE = 0,
+	COLUMN_ARTIST,
+	COLUMN_ALBUM,
+	COLUMN_LENGTH
+} GimmixColumnType;
 
 typedef enum {
 	SONG = 1,
@@ -35,9 +43,19 @@ typedef enum {
 	PLAYLIST
 } GimmixFileType;
 
+enum { TARGET_STRING, TARGET_ROOTWIN };
+  	GtkTargetEntry targetentries[] =
+    	{
+     		{ "text/plain", 0, TARGET_STRING },
+      		{ "application/x-rootwindow-drop", 0, TARGET_ROOTWIN }
+	};
+	guint n_targets = sizeof(targetentries) / sizeof(targetentries[0]);
+	
 extern MpdObj		*gmo;
 extern GladeXML 	*xml;
 extern ConfigFile	conf;
+extern GtkWidget	*main_window;
+
 
 static gchar *invalid_dir_error = "You have specified an invalid music directory. Do you want to specify the correct music directory now ?";
 
@@ -63,19 +81,124 @@ GtkWidget		*pls_button_add;
 GtkWidget		*pls_button_save;
 GtkWidget		*pls_playlists_box;
 
+/* Tree view columns */
+static GtkWidget	*cpl_tvw_title_column;
+static GtkWidget	*cpl_tvw_artist_column;
+static GtkWidget	*cpl_tvw_album_column;
+static GtkWidget	*cpl_tvw_length_column;
+
 extern GtkWidget	*tag_editor_window;
 
 gchar			*loaded_playlist;
 
-void
-onDragDataRecived(GtkWidget *widget,
-		  GdkDragContext *context,
-		  int x, int y,
-		  GtkSelectionData *seldata,
-		  guint info, guint time,
-		  gpointer userdata)
+static void
+on_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y, 
+	GtkSelectionData *selection_data, guint target_type, guint time, 
+	gpointer data)
 {
-	g_print ("Dropped.\n");
+	gchar		*path;
+	gboolean	dnd_success = FALSE;
+	gboolean	delete_selection_data = FALSE;
+
+	if((selection_data != NULL) && (selection_data->length >= 0))
+        {
+		switch (target_type)
+		{
+			case TARGET_STRING:
+			{
+				path = (gchar*) selection_data->data;
+				mpd_playlist_queue_add (gmo, path);
+				mpd_playlist_queue_commit (gmo);
+				dnd_success = TRUE;
+				break;
+			}
+                                        
+			default:
+			{
+				g_print ("default");
+			}
+                }
+        }
+
+	if (dnd_success == FALSE)
+	{
+		g_print ("DnD data transfer failed!\n");
+	}
+        gtk_drag_finish (context, dnd_success, delete_selection_data, time);
+	
+	return;
+}
+
+static void
+on_drag_data_get (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data,
+	guint target_type, guint time, gpointer user_data)
+{
+	GtkTreeSelection	*selection = NULL;
+	GtkTreeModel		*model = NULL;
+	GList			*list = NULL;
+	GimmixFileType		type = -1;
+	gchar			*path = NULL;
+	GtkTreeIter		iter;
+
+        g_assert (selection_data != NULL);
+        
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(library_treeview));
+	list = gtk_tree_selection_get_selected_rows (library_selection, &model);
+	while (list != NULL)
+	{
+		gtk_tree_model_get_iter (model, &iter, list->data);
+		gtk_tree_model_get (model, &iter, 2, &path, 3, &type, -1);
+		
+		if (type == DIR || type == SONG)
+		{
+			switch (target_type)
+			{
+				case TARGET_STRING:	
+				{
+					gtk_selection_data_set (selection_data, selection_data-> target, 8, (guchar*) path, strlen (path));
+					break;
+				}
+                	
+				case TARGET_ROOTWIN:
+				{
+					g_print ("Dropped on the root window!\n");
+					break;
+				}
+                	
+				default:
+				{
+					g_assert_not_reached ();
+				}
+			}
+			g_free (path);
+		}
+		list = g_list_next (list);
+	}
+
+	return;
+}
+
+static gboolean
+on_drag_drop (GtkWidget *widget, GdkDragContext *context, gint x, gint y, guint time,
+	gpointer user_data)
+{
+	gboolean ret = TRUE;
+
+        /* If a target is offerred */
+        if (context->targets)
+        {
+		/* Request the data from the source. */
+		gtk_drag_get_data (widget,
+				context,
+				TARGET_STRING,
+				time);
+        }
+        else
+        {
+                ret = FALSE;
+        }
+        
+        return  ret;
 }
 
 static void		gimmix_search_init (void);
@@ -92,7 +215,6 @@ static void		gimmix_display_total_playlist_time (MpdObj *);
 /* Callbacks */
 /* Current playlist callbacks */
 static void		cb_current_playlist_double_click (GtkTreeView *treeview);
-static void		cb_current_playlist_right_click (GtkTreeView *treeview, GdkEventButton *event);
 static void		cb_repeat_menu_toggled (GtkCheckMenuItem *item, gpointer data);
 static void		cb_shuffle_menu_toggled (GtkCheckMenuItem *item, gpointer data);
 static void		cb_current_playlist_delete_press (GtkWidget *widget, GdkEventKey *event, gpointer data);
@@ -119,7 +241,7 @@ static void		gimmix_update_playlists_treeview (void);
 static void		gimmix_playlist_save_dialog_show (void);
 static void		cb_gimmix_playlist_save_response (GtkDialog *dlg, gint arg1, gpointer dialog);
 static void		cb_playlists_right_click (GtkTreeView *treeview, GdkEventButton *event);
-static bool		cb_all_playlist_button_press (GtkTreeView *treeview, GdkEventButton *event);
+static bool		cb_all_playlist_button_press (GtkTreeView *treeview, GdkEventButton *event, gpointer data);
 static void		cb_gimmix_playlist_remove ();
 static void		cb_gimmix_playlist_load ();
 static void		cb_playlists_delete_press (GtkWidget *widget, GdkEventKey *event, gpointer data);
@@ -138,7 +260,7 @@ gimmix_playlist_search_widgets_init (void)
 }
 
 void
-gimmix_playlist_widgets_init (void)
+gimmix_playlist_setup_current_playlist_tvw (void)
 {
 	GtkTreeModel		*current_playlist_model;
 	GtkListStore		*current_playlist_store;
@@ -151,39 +273,114 @@ gimmix_playlist_widgets_init (void)
 	library_window = glade_xml_get_widget (xml,"library_window");
 	
 	current_playlist_renderer = gtk_cell_renderer_text_new ();
+	g_object_set (G_OBJECT(current_playlist_renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 	current_playlist_column = gtk_tree_view_column_new_with_attributes (_("Title"),
 										current_playlist_renderer,
 										"markup", 0,
 										NULL);
 	gtk_tree_view_column_set_resizable (current_playlist_column, TRUE);
-	gtk_tree_view_column_set_min_width (current_playlist_column, 200);
+	gtk_tree_view_column_set_clickable (current_playlist_column, TRUE);
+	//gtk_tree_view_column_set_min_width (current_playlist_column, 200);
 	g_object_set (G_OBJECT(current_playlist_column), "expand", TRUE, "spacing", 4, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(current_playlist_treeview), current_playlist_column);
+	cpl_tvw_title_column = (GtkWidget*)current_playlist_column;
+	
+	current_playlist_renderer = gtk_cell_renderer_text_new ();
+	g_object_set (G_OBJECT(current_playlist_renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	current_playlist_column = gtk_tree_view_column_new_with_attributes (_("Artist"),
+										current_playlist_renderer,
+										"markup", 3,
+										NULL);
+	gtk_tree_view_column_set_resizable (current_playlist_column, TRUE);
+	gtk_tree_view_column_set_clickable (current_playlist_column, TRUE);
+	//gtk_tree_view_column_set_min_width (current_playlist_column, 100);
+	g_object_set (G_OBJECT(current_playlist_column), "expand", TRUE, "spacing", 4, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW(current_playlist_treeview), current_playlist_column);
+	cpl_tvw_artist_column = (GtkWidget*)current_playlist_column;
+	
+	current_playlist_renderer = gtk_cell_renderer_text_new ();
+	g_object_set (G_OBJECT(current_playlist_renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	current_playlist_column = gtk_tree_view_column_new_with_attributes (_("Album"),
+										current_playlist_renderer,
+										"markup", 4,
+										NULL);
+	gtk_tree_view_column_set_resizable (current_playlist_column, TRUE);
+	gtk_tree_view_column_set_clickable (current_playlist_column, TRUE);
+	//gtk_tree_view_column_set_min_width (current_playlist_column, 100);
+	g_object_set (G_OBJECT(current_playlist_column), "expand", TRUE, "spacing", 4, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW(current_playlist_treeview), current_playlist_column);
+	cpl_tvw_album_column = (GtkWidget*)current_playlist_column;
 							
 	current_playlist_renderer = gtk_cell_renderer_text_new ();
 	current_playlist_column = gtk_tree_view_column_new_with_attributes (_("Length"),
 										current_playlist_renderer,
-										"markup", 3,
+										"markup", 5,
 										NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW(current_playlist_treeview), current_playlist_column);
-
-	current_playlist_store = gtk_list_store_new (4,
+	cpl_tvw_length_column = (GtkWidget*)current_playlist_column;
+	
+	current_playlist_store = gtk_list_store_new (6,
 						G_TYPE_STRING, 	/* name (0) */
 						G_TYPE_STRING, 	/* path (1) */
-						G_TYPE_INT,		/* id	(2) */
-						G_TYPE_STRING); /* length (3) */
+						G_TYPE_INT,	/* id	(2) */
+						G_TYPE_STRING,  /* artist (3) */
+						G_TYPE_STRING,  /* album (4) */
+						G_TYPE_STRING); /* length (5) */
 	current_playlist_model	= GTK_TREE_MODEL (current_playlist_store);
 	current_playlist_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(current_playlist_treeview));
 	gtk_tree_selection_set_mode (current_playlist_selection, GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (current_playlist_treeview), current_playlist_model);
+	
+	/* Drag and Drop */
+	gtk_drag_dest_set(GTK_WIDGET(current_playlist_treeview),
+				GTK_DEST_DEFAULT_ALL,
+				targetentries, n_targets - 1,
+				GDK_ACTION_COPY|GDK_ACTION_MOVE);
+	
+	gtk_drag_source_set (GTK_WIDGET(library_treeview),
+				GDK_BUTTON1_MASK,
+				targetentries, n_targets-1,
+				GDK_ACTION_COPY);
+
+	g_signal_connect(GTK_WIDGET(current_playlist_treeview),
+			"drag_data_received",
+			G_CALLBACK(on_drag_data_received),
+			NULL);
+	g_signal_connect (current_playlist_treeview,
+			"drag-drop",
+			G_CALLBACK (on_drag_drop),
+			NULL);
+	g_signal_connect (library_treeview,
+			"drag-data-get",
+			G_CALLBACK (on_drag_data_get),
+			NULL);
+			
+	/* load default values */
+	gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(cpl_tvw_title_column),
+					gimmix_config_get_bool("pl_column_title_show"));
+	gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(cpl_tvw_artist_column),
+					gimmix_config_get_bool("pl_column_artist_show"));
+	gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(cpl_tvw_album_column),
+					gimmix_config_get_bool("pl_column_album_show"));
+	gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(cpl_tvw_length_column),
+					gimmix_config_get_bool("pl_column_length_show"));
+			
+	g_object_unref (current_playlist_model);
+}
+
+void
+gimmix_playlist_widgets_init (void)
+{
+	gimmix_playlist_setup_current_playlist_tvw ();
+	
 	gimmix_statusbar = glade_xml_get_widget (xml, "gimmix_status");
 	gimmix_statusbox = glade_xml_get_widget (xml, "gimmix_statusbox");
 	
 	g_signal_connect (current_playlist_treeview, "row-activated", G_CALLBACK(cb_current_playlist_double_click), NULL);
-	g_signal_connect (current_playlist_treeview, "button-press-event", G_CALLBACK(cb_all_playlist_button_press), NULL);
-	g_signal_connect (current_playlist_treeview, "button-release-event", G_CALLBACK (cb_current_playlist_right_click), NULL);
+	g_signal_connect (current_playlist_treeview, "button-press-event", G_CALLBACK(cb_all_playlist_button_press), (gpointer)TRUE);
+	//g_signal_connect (current_playlist_treeview, "button-release-event", G_CALLBACK (cb_current_playlist_right_click), NULL);
 	g_signal_connect (current_playlist_treeview, "key_release_event", G_CALLBACK (cb_current_playlist_delete_press), NULL);
-	g_object_unref (current_playlist_model);
+	
 	g_signal_connect (playlists_treeview, "button-release-event", G_CALLBACK (cb_playlists_right_click), NULL);
 	g_signal_connect (playlists_treeview, "key_release_event", G_CALLBACK (cb_playlists_delete_press), NULL);
 	
@@ -214,27 +411,8 @@ gimmix_playlist_widgets_init (void)
 	current_pl_window = glade_xml_get_widget (xml, "current_pl_window");
 	pls_playlist_window = glade_xml_get_widget (xml, "pls_playlist_window");
 
-	/* Drag and Drop */
-	enum { TARGET_STRING, TARGET_URI };
-  	GtkTargetEntry targetentries[] =
-    	{
-     		{ "STRING",        0, TARGET_STRING },
-      		{ "text/plain",    0, TARGET_STRING },      		
-		{ "text/uri-list", 0, TARGET_URI },
-	};
+	
 
-	gtk_drag_dest_set(GTK_WIDGET(current_playlist_treeview),
-		    GTK_DEST_DEFAULT_ALL,
-		    targetentries, 3,
-		    GDK_ACTION_COPY|GDK_ACTION_MOVE);
-	gtk_drag_source_set(GTK_WIDGET(current_playlist_treeview),
-		    GDK_BUTTON1_MASK,
-		    targetentries, 3,
-		    GDK_ACTION_COPY|GDK_ACTION_MOVE);
-	g_signal_connect(GTK_WIDGET(current_playlist_treeview),
-		   "drag_data_received",
-		   G_CALLBACK(onDragDataRecived),
-		   current_playlist_store);
 	loaded_playlist = NULL;
 	
 	gimmix_playlist_search_widgets_init ();
@@ -267,9 +445,9 @@ gimmix_playlist_enable_controls (void)
 static void
 gimmix_search_init (void)
 {
-	if (strncasecmp(cfg_get_key_value(conf, "enable_search"), "true", 4) != 0)
+	if (!gimmix_config_get_bool("enable_search"))
 		gtk_widget_hide (search_box);
-		
+
 	return;
 }
 
@@ -294,7 +472,7 @@ gimmix_update_current_playlist (MpdObj *mo, MpdData *pdata)
 	gint		current_song_id = -1;
 	MpdData		*data = pdata;
 
-	if (mo!=NULL && mpd_check_connected(mo))
+	if (data!=NULL && mpd_check_connected(mo))
 	{
 		new = mpd_playlist_get_playlist_id (mo);
 		current_song_id = mpd_player_get_current_song_id (mo);
@@ -308,7 +486,9 @@ gimmix_update_current_playlist (MpdObj *mo, MpdData *pdata)
 	
 	while (data != NULL)
 	{
-		gchar 	*title;
+		gchar 	*title = NULL;
+		gchar	*artist = NULL;
+		gchar	*album = NULL;
 		gchar 	*ti;
 		gchar	time[15];
 		
@@ -316,10 +496,12 @@ gimmix_update_current_playlist (MpdObj *mo, MpdData *pdata)
 		{
 			if (data->song->title != NULL)
 			{
+				/*
 				if (data->song->artist)
 				title = g_markup_printf_escaped ("<span size=\"medium\"weight=\"bold\">%s - %s</span>", data->song->artist, data->song->title);
 				else
-					title = g_markup_printf_escaped ("<span size=\"medium\"weight=\"bold\">%s</span>", data->song->title);
+				*/
+				title = g_markup_printf_escaped ("<span size=\"medium\"weight=\"bold\">%s</span>", data->song->title);
 			}
 			else
 			{
@@ -328,6 +510,14 @@ gimmix_update_current_playlist (MpdObj *mo, MpdData *pdata)
 				title = g_markup_printf_escaped ("<span size=\"medium\"weight=\"bold\">%s</span>", file);
 				g_free (file);
 			}
+			if (data->song->artist != NULL)
+			{
+				artist = g_markup_printf_escaped ("<span size=\"medium\"weight=\"bold\">%s</span>", data->song->artist);
+			}
+			if (data->song->album != NULL)
+			{
+				album = g_markup_printf_escaped ("<span size=\"medium\"weight=\"bold\">%s</span>", data->song->album);
+			}
 			gimmix_get_total_time_for_song (mo, data->song, time);
 			ti = g_markup_printf_escaped ("<span size=\"medium\" weight=\"bold\">%s</span>", time);
 		}
@@ -335,16 +525,24 @@ gimmix_update_current_playlist (MpdObj *mo, MpdData *pdata)
 		{	
 			if (data->song->title != NULL)
 			{
-				if (data->song->artist)
+				/*if (data->song->artist)
 				title = g_markup_printf_escaped ("%s - %s", data->song->artist, data->song->title);
-				else
-					title = g_markup_printf_escaped ("%s", data->song->title);
+				else*/
+				title = g_markup_printf_escaped ("%s", data->song->title);
 			}
 			else
 			{
 				title = g_markup_printf_escaped (g_path_get_basename(data->song->file));
 				gimmix_strip_file_ext (title);
 			}
+			if (data->song->artist)
+			{
+				artist = g_markup_printf_escaped ("%s", data->song->artist);
+			}
+			if (data->song->album)
+			{
+				album = g_markup_printf_escaped ("%s", data->song->album);
+			}
 			gimmix_get_total_time_for_song (mo, data->song, time);
 			ti = NULL;
 		}
@@ -353,14 +551,18 @@ gimmix_update_current_playlist (MpdObj *mo, MpdData *pdata)
 		gtk_list_store_set (current_playlist_store, 
 							&current_playlist_iter,
 							0, title,
-							3, (ti!=NULL) ? ti : time,
+							5, (ti!=NULL) ? ti : time,
 							1, data->song->file,
 							2, data->song->id,
+							3, artist,
+							4, album,
 							-1);
 		data = mpd_data_get_next (data);
 		if (ti)
 		g_free (ti);
 		g_free (title);
+		g_free (album);
+		g_free (artist);
 	}
 	gtk_tree_view_set_model (GTK_TREE_VIEW (current_playlist_treeview), GTK_TREE_MODEL(current_playlist_store));
 	gimmix_display_total_playlist_time (mo);
@@ -514,24 +716,26 @@ gimmix_library_and_playlists_populate (void)
 				title = g_path_get_basename (data->song->file);
 				gimmix_strip_file_ext (title);
 			}
-			gtk_list_store_set (dir_store, &dir_iter,
-								0, song_pixbuf,
-								1, title,
-								2, data->song->file,
-								3, SONG,
-								-1);
+			gtk_list_store_set (dir_store,
+						&dir_iter,
+						0, song_pixbuf,
+						1, title,
+						2, data->song->file,
+						3, SONG,
+						-1);
 			g_free (title);
 		}
 		else if (data->type == MPD_DATA_TYPE_PLAYLIST)
 		{
-			gchar *name;
-			
-			name = data->playlist;
+			gchar *name = NULL;
+			name = ((mpd_PlaylistFile*)data->playlist)->path;
+			//g_print ("ADDING %s\n", name);
 			gtk_list_store_append (pls_store, &pls_iter);
-			gtk_list_store_set (pls_store, &pls_iter,
-								0, pls_pixbuf,
-								1, name,
-								-1);
+			gtk_list_store_set (pls_store,
+						&pls_iter,
+						0, pls_pixbuf,
+						1, name,
+						-1);
 		}
 	}
 
@@ -551,11 +755,11 @@ gimmix_library_and_playlists_populate (void)
 
 	g_object_unref (dir_model);
 	g_object_unref (pls_model);
-	/*
+	
 	g_object_unref (dir_pixbuf);
 	g_object_unref (song_pixbuf);
 	g_object_unref (pls_pixbuf);
-	*/
+	
 	return;
 }
 
@@ -621,7 +825,7 @@ gimmix_library_search (gint type, gchar *text)
 	song_pixbuf = gdk_pixbuf_new_from_file_at_size (path, 12, 12, NULL);
 	g_free (path);
 
-	for (data; data!=NULL; data = mpd_data_get_next (data))
+	for (; data!=NULL; data = mpd_data_get_next (data))
 	{
 		if (data->type == MPD_DATA_TYPE_SONG)
 		{
@@ -701,12 +905,19 @@ cb_current_playlist_double_click (GtkTreeView *treeview)
 
 /* This keeps multiple items selected in case of a right click */
 static bool
-cb_all_playlist_button_press (GtkTreeView *treeview, GdkEventButton *event)
+cb_all_playlist_button_press (GtkTreeView *treeview, GdkEventButton *event, gpointer data)
 {
-	GtkTreeSelection *selection;
+	GtkTreeSelection	*selection;
+	gboolean		current_playlist = (gboolean) data;
 	
 	if (event->button == 3)
 	{
+		/* check if it's a right click in current playlist treeview */
+		if (current_playlist)
+		{
+			/* if yes, then also show the popup */
+			gimmix_current_playlist_popup_menu ();
+		}
 		GtkTreePath *path;
 		
 		if (gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, &path, NULL, NULL, NULL))
@@ -715,7 +926,7 @@ cb_all_playlist_button_press (GtkTreeView *treeview, GdkEventButton *event)
 			bool sel = gtk_tree_selection_path_is_selected (selection, path);
 			gtk_tree_path_free (path);
 			
-			if (sel)
+			if (sel && current_playlist)
 				return TRUE;
 		}
 	}
@@ -755,7 +966,7 @@ cb_library_dir_activated (gpointer data)
 		g_free (path);
 	}
 	
-	if ((strncasecmp(cfg_get_key_value(conf, "play_on_add"), "true", 4) == 0) && (added == true))
+	if (gimmix_config_get_bool("play_on_add") && (added == true))
 	{
 		/* If we're adding, there might already be a song playing. */
 		int state;
@@ -827,7 +1038,7 @@ cb_library_popup_add_clicked (GtkWidget *widget, gpointer data)
 	}
 	
 	mpd_playlist_queue_commit (gmo);
-	if (strncasecmp(cfg_get_key_value(conf, "play_on_add"), "true", 4) == 0)
+	if (gimmix_config_get_bool("play_on_add"))
 	{
 		/* If we're adding, there might already be a song playing. */
 		int state;
@@ -848,7 +1059,8 @@ cb_library_popup_add_clicked (GtkWidget *widget, gpointer data)
 	return;
 }
 
-static void cd_library_popup_replace_clicked (GtkWidget *widget, gpointer data)
+static void
+cd_library_popup_replace_clicked (GtkWidget *widget, gpointer data)
 {
 
 	GtkTreeModel 		*model;
@@ -903,7 +1115,7 @@ static void cd_library_popup_replace_clicked (GtkWidget *widget, gpointer data)
 
 	mpd_playlist_queue_commit (gmo);
 	
-	if (strncasecmp(cfg_get_key_value(conf, "play_on_add"), "true", 4) == 0)
+	if (gimmix_config_get_bool("play_on_add"))
 	{
 		gimmix_play(gmo);
 	}
@@ -997,15 +1209,15 @@ gimmix_update_playlists_treeview (void)
 static void
 gimmix_update_library_with_dir (gchar *dir)
 {
-	GtkTreeModel		*directory_model;
-	GtkListStore		*dir_store;
-	GtkTreeIter			dir_iter;
-	GdkPixbuf			*dir_pixbuf;
-	GdkPixbuf			*song_pixbuf;
-	MpdData				*data;
-	gchar				*parent;
-	gchar				*path;
-	gchar				*directory;
+	GtkTreeModel	*directory_model;
+	GtkListStore	*dir_store;
+	GtkTreeIter	dir_iter;
+	GdkPixbuf	*dir_pixbuf;
+	GdkPixbuf	*song_pixbuf;
+	MpdData		*data;
+	gchar		*parent;
+	gchar		*path;
+	gchar		*directory;
 	
 	directory_model = gtk_tree_view_get_model (GTK_TREE_VIEW (library_treeview));
 	dir_store 	= GTK_LIST_STORE (directory_model);
@@ -1017,9 +1229,9 @@ gimmix_update_library_with_dir (gchar *dir)
 	gtk_list_store_clear (dir_store);
 
 	dir_pixbuf 	= gtk_widget_render_icon (GTK_WIDGET(library_treeview),
-										GTK_STOCK_DIRECTORY,
-										GTK_ICON_SIZE_BUTTON,
-										NULL);
+							GTK_STOCK_DIRECTORY,
+							GTK_ICON_SIZE_BUTTON,
+							NULL);
 	path = gimmix_get_full_image_path (GIMMIX_MEDIA_ICON);
 	song_pixbuf = gdk_pixbuf_new_from_file_at_size (path, 16, 16, NULL);
 	g_free (path);
@@ -1029,11 +1241,11 @@ gimmix_update_library_with_dir (gchar *dir)
 		parent = gimmix_path_get_parent_dir (dir);
 		gtk_list_store_append (dir_store, &dir_iter);
 		gtk_list_store_set (dir_store, &dir_iter,
-								0, dir_pixbuf,
-								1, "..",
-								2, parent,
-								3, DIR,
-								-1);
+					0, dir_pixbuf,
+					1, "..",
+					2, parent,
+					3, DIR,
+					-1);
 		g_free (parent);
 	}
 	
@@ -1089,17 +1301,6 @@ gimmix_update_library_with_dir (gchar *dir)
 	return;
 }
 
-static void
-cb_current_playlist_right_click (GtkTreeView *treeview, GdkEventButton *event)
-{	
-	if (event->button == 3) /* If right click */
-	{
-		gimmix_current_playlist_popup_menu ();
-	}
-	
-	return;
-}
-
 static void
 cb_library_right_click (GtkTreeView *treeview, GdkEventButton *event)
 {	
@@ -1179,16 +1380,28 @@ gimmix_current_playlist_song_info (void)
 	gtk_tree_model_get_iter (model, &iter, list->data);
 	gtk_tree_model_get (model, &iter, 1, &path, -1);
 	
+	#ifdef HAVE_TAGEDITOR
 	song_path = g_strdup_printf ("%s/%s", cfg_get_key_value(conf, "music_directory"), path);
 	if (gimmix_tag_editor_populate (song_path))
 	{	
 		gtk_widget_show (tag_editor_window);
 	}
 	else
+	{
 		gimmix_tag_editor_error (invalid_dir_error);
-		
-	g_free (path);
+	}
 	g_free (song_path);
+	#else
+	if (gimmix_tag_editor_populate (mpd_playlist_get_current_song(gmo)))
+	{
+		gtk_widget_show (tag_editor_window);
+	}
+	else
+	{
+		gimmix_tag_editor_error (_("An error occurred while trying to get song information. Please try again."));
+	}
+	#endif	
+	g_free (path);
 	
 	/* free the list */
 	g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
@@ -1292,10 +1505,61 @@ gimmix_current_playlist_clear (void)
 	return;
 }
 
+static void
+cb_gimmix_playlist_column_show_toggled (GtkCheckMenuItem *menu_item, gpointer data)
+{
+	gint		column = (gint) data;
+	gchar		*kval;
+	gboolean	value;
+
+	if (gtk_check_menu_item_get_active(menu_item))
+	{
+		kval = "true";
+		value = TRUE;
+	}
+	else
+	{
+		kval = "false";
+		value = FALSE;
+	}
+	
+	switch (column)
+	{
+		case COLUMN_TITLE:
+		{
+			cfg_add_key (&conf, "pl_column_title_show", kval);
+			gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(cpl_tvw_title_column), value);
+			break;
+		}
+		case COLUMN_ARTIST:
+		{
+			cfg_add_key (&conf, "pl_column_artist_show", kval);
+			gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(cpl_tvw_artist_column), value);
+			break;
+		}
+		case COLUMN_ALBUM:
+		{
+			cfg_add_key (&conf, "pl_column_album_show", kval);
+			gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(cpl_tvw_album_column), value);
+			break;
+		}
+		case COLUMN_LENGTH:
+		{
+			cfg_add_key (&conf, "pl_column_length_show", kval);
+			gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN(cpl_tvw_length_column), value);
+			break;
+		}
+		default:
+		break;
+	}
+	gimmix_config_save ();
+}
+
 static void
 gimmix_current_playlist_popup_menu (void)
 {
 	GtkWidget *menu, *menu_item, *image;
+	GtkWidget *submenu = NULL;
 
 	menu = gtk_menu_new ();
 	image = gtk_image_new_from_stock ("gtk-cut", GTK_ICON_SIZE_MENU); 
@@ -1353,14 +1617,50 @@ gimmix_current_playlist_popup_menu (void)
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 	gtk_widget_show (menu_item);
 
+	submenu = gtk_menu_new ();
+	menu_item = gtk_check_menu_item_new_with_label (_("Title"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu_item);
+	if (gimmix_config_get_bool("pl_column_title_show"))
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+	g_signal_connect (G_OBJECT (menu_item), "toggled", G_CALLBACK (cb_gimmix_playlist_column_show_toggled), (gpointer)COLUMN_TITLE);
+	gtk_widget_show (menu_item);
+	
+	menu_item = gtk_check_menu_item_new_with_label (_("Artist"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu_item);
+	if (gimmix_config_get_bool("pl_column_artist_show"))
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+	g_signal_connect (G_OBJECT (menu_item), "toggled", G_CALLBACK (cb_gimmix_playlist_column_show_toggled), (gpointer)COLUMN_ARTIST);
+	gtk_widget_show (menu_item);
+	
+	menu_item = gtk_check_menu_item_new_with_label (_("Album"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu_item);
+	if (gimmix_config_get_bool("pl_column_album_show"))
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+	g_signal_connect (G_OBJECT (menu_item), "toggled", G_CALLBACK (cb_gimmix_playlist_column_show_toggled), (gpointer)COLUMN_ALBUM);
+	gtk_widget_show (menu_item);
+	
+	menu_item = gtk_check_menu_item_new_with_label (_("Length"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu_item);
+	if (gimmix_config_get_bool("pl_column_length_show"))
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+	g_signal_connect (G_OBJECT (menu_item), "toggled", G_CALLBACK (cb_gimmix_playlist_column_show_toggled), (gpointer)COLUMN_LENGTH);
+	gtk_widget_show (menu_item);
+	
+	menu_item = gtk_image_menu_item_new_with_label (_("Columns"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM(menu_item), GTK_WIDGET(get_image ("gtk-edit", GTK_ICON_SIZE_MENU)));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+	gtk_widget_show (menu_item);
+	
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM(menu_item), submenu);
+	
 	gtk_widget_show (menu);
 	gtk_menu_popup (GTK_MENU(menu),
-					NULL,
-					NULL,
-					NULL,
-					NULL,
-					3,
-					gtk_get_current_event_time());
+			NULL,
+			NULL,
+			NULL,
+			NULL,
+			0,
+			gtk_get_current_event_time());
 	
 	return;
 }
@@ -1368,10 +1668,10 @@ gimmix_current_playlist_popup_menu (void)
 static void
 gimmix_library_popup_menu (void)
 {
-	GtkWidget 			*menu;
-	GtkWidget 			*menu_item;
-	GtkWidget			*image;
-	GtkWidget			*replace_icon;
+	GtkWidget 	*menu;
+	GtkWidget 	*menu_item;
+	GtkWidget	*image;
+	GtkWidget	*replace_icon;
 
 	menu = gtk_menu_new ();
 	image = gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_MENU);
@@ -1408,14 +1708,13 @@ gimmix_library_popup_menu (void)
 	
 	gtk_widget_show (menu);
 	gtk_menu_popup (GTK_MENU(menu),
-					NULL,
-					NULL,
-					NULL,
-					NULL,
-					3,
-					gtk_get_current_event_time());
-					
-	return;
+			NULL,
+			NULL,
+			NULL,
+			NULL,
+			3,
+			gtk_get_current_event_time());
+			return;
 }
 
 static void
@@ -1557,6 +1856,7 @@ gimmix_playlist_save_dialog_show (void)
                              "response", 
                              G_CALLBACK (cb_gimmix_playlist_save_response),
                              dialog);
+	g_signal_connect (dialog, "delete-event", G_CALLBACK(gtk_widget_destroy), dialog);
 	gtk_misc_set_padding (GTK_MISC(label), 5, 5);
 	gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER((GTK_DIALOG(dialog))->vbox), 10);
