@@ -1,7 +1,7 @@
 /*
  * gimmix-tagedit.c
  *
- * Copyright (C) 2006-2008 Priyank Gosalia
+ * Copyright (C) 2006-2009 Priyank Gosalia
  *
  * Gimmix is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -29,24 +29,29 @@ extern MpdObj 		*gmo;
 extern GladeXML 	*xml;
 extern ConfigFile	conf;
 
+#ifdef HAVE_TAGEDITOR
 TagLib_File 	*file = NULL;
 TagLib_Tag 	*tag = NULL;
+#endif
 
-GtkWidget	*tag_title;
-GtkWidget	*tag_file;
-GtkWidget	*tag_artist;
-GtkWidget	*tag_album;
-GtkWidget	*tag_comment;
-GtkWidget	*tag_track_spin;
-GtkWidget	*tag_year_spin;
-GtkWidget	*tag_genre;
-GtkWidget	*tag_info_bitrate;
-GtkWidget	*tag_info_channels;
-GtkWidget	*tag_info_length;
+static GtkWidget	*tag_title;
+static GtkWidget	*tag_file;
+static GtkWidget	*tag_artist;
+static GtkWidget	*tag_album;
+static GtkWidget	*tag_comment;
+static GtkWidget	*tag_track_spin;
+static GtkWidget	*tag_year_spin;
+static GtkWidget	*tag_genre;
+static GtkWidget	*tag_info_bitrate;
+static GtkWidget	*tag_info_channels;
+static GtkWidget	*tag_info_length;
 GtkWidget	*tag_editor_window;
+static GtkWidget	*tag_editor_cover_image;
 
+#ifdef HAVE_TAGEDITOR
 /* Save action */
 static void	gimmix_tag_editor_save (GtkWidget *button, gpointer data);
+#endif
 
 /* Close action */
 static void gimmix_tag_editor_close (GtkWidget *widget, gpointer data);
@@ -54,8 +59,10 @@ static void gimmix_tag_editor_close (GtkWidget *widget, gpointer data);
 /* error dialog callback */
 static void cb_gimmix_tag_editor_error_response (GtkDialog *dialog, gint arg1, gpointer data);
 
+#ifdef HAVE_TAGEDITOR
 const gchar *dir_error = "You have specified an invalid music directory. Do you want to specify the correct music directory now ?";
-	
+#endif
+
 void
 gimmix_tag_editor_widgets_init (void)
 {
@@ -63,7 +70,7 @@ gimmix_tag_editor_widgets_init (void)
 	
 	tag_editor_window = glade_xml_get_widget (xml, "tag_editor_window");
 	tag_title = glade_xml_get_widget (xml,"entry_title");
-	tag_file = glade_xml_get_widget (xml,"label_filename");
+	tag_file = glade_xml_get_widget (xml,"entry_filename");
 	tag_artist = glade_xml_get_widget (xml,"entry_artist");
 	tag_album = glade_xml_get_widget (xml,"entry_album");
 	tag_comment = glade_xml_get_widget (xml,"entry_comment");
@@ -73,31 +80,48 @@ gimmix_tag_editor_widgets_init (void)
 	tag_info_length = glade_xml_get_widget (xml, "info_length");
 	tag_info_channels = glade_xml_get_widget (xml, "info_channels");
 	tag_info_bitrate = glade_xml_get_widget (xml, "info_bitrate");
+	tag_editor_cover_image = glade_xml_get_widget (xml, "gimmix_tagedit_cover_image");
 	
 	widget = glade_xml_get_widget (xml, "tag_editor_save");
+	#ifdef HAVE_TAGEDITOR
 	g_signal_connect (G_OBJECT(widget), "clicked", G_CALLBACK(gimmix_tag_editor_save), NULL);
-	
+	#else
+	gtk_widget_set_sensitive (widget, FALSE);
+	#endif
+
 	widget = glade_xml_get_widget (xml, "tag_editor_close");
-	g_signal_connect (G_OBJECT(widget), "clicked", G_CALLBACK(gimmix_tag_editor_close), NULL);
-	g_signal_connect (G_OBJECT(tag_editor_window), "delete-event", G_CALLBACK(gtk_widget_hide_on_delete), NULL);
+	g_signal_connect (G_OBJECT(widget),
+				"clicked",
+				G_CALLBACK(gimmix_tag_editor_close),
+				NULL);
+	g_signal_connect (G_OBJECT(tag_editor_window),
+				"delete-event",
+				G_CALLBACK(gtk_widget_hide_on_delete),
+				NULL);
 	
 	return;
 }
 
 /* Load the tag editor */
 gboolean
-gimmix_tag_editor_populate (const gchar *song)
+gimmix_tag_editor_populate (const void *song)
 {
 	GtkTreeModel 	*genre_model;
 	gchar 		*info;
-	gint 		min;
-	gint		sec;
 	gint 		n;
-	const TagLib_AudioProperties *properties;
+	guint		year = 0;
+	guint		track = 0;
+	gchar		*title = NULL;
+	gchar		*artist = NULL;
+	gchar		*album = NULL;
+	gchar		*genre = NULL;
+	gchar		*comment = NULL;
 	
 	if (!song)
 		return FALSE;
 	
+	#ifdef HAVE_TAGEDITOR
+	const TagLib_AudioProperties *properties;
 	if (!g_file_test (song, G_FILE_TEST_EXISTS))
 		return FALSE;
 		
@@ -109,36 +133,108 @@ gimmix_tag_editor_populate (const gchar *song)
 	
 	tag = taglib_file_tag (file);
 	properties = taglib_file_audioproperties (file);
+	#else
+	mpd_Song *foo = (mpd_Song*) g_malloc0 (sizeof(mpd_Song));
+	memcpy (foo, song, sizeof(mpd_Song));
+	#endif
 	
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON(tag_year_spin), taglib_tag_year(tag));
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON(tag_track_spin), taglib_tag_track(tag));
-	gtk_label_set_text (GTK_LABEL(tag_file), song);
-	gtk_entry_set_text (GTK_ENTRY(tag_title), g_strstrip(taglib_tag_title(tag)));
-	gtk_entry_set_text (GTK_ENTRY(tag_artist), g_strstrip(taglib_tag_artist(tag)));
-	gtk_entry_set_text (GTK_ENTRY(tag_album), g_strstrip(taglib_tag_album(tag)));
-	gtk_entry_set_text (GTK_ENTRY(tag_comment), g_strstrip(taglib_tag_comment(tag)));
+	#ifdef HAVE_TAGEDITOR
+	year = taglib_tag_year (tag);
+	track = taglib_tag_track (tag);
+	title = g_strstrip (taglib_tag_title(tag));
+	artist = g_strstrip (taglib_tag_artist(tag));
+	album = g_strstrip (taglib_tag_album(tag));
+	comment = g_strstrip (taglib_tag_comment(tag));
+	genre = taglib_tag_genre (tag);
+	#else
+	if (foo->date)
+	{
+		year = atoi (foo->date);
+	}
+	if (foo->track)
+	{
+		track = atoi (foo->track);
+	}
+	title = foo->title;
+	artist = foo->artist;
+	album = foo->album;
+	comment = foo->comment;
+	genre = foo->genre;
+	#endif
 	
-	gtk_combo_box_append_text (GTK_COMBO_BOX(tag_genre), taglib_tag_genre(tag));
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(tag_year_spin), year);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(tag_track_spin), track);
+
+	gtk_entry_set_text (GTK_ENTRY(tag_file),
+				#ifdef HAVE_TAGEDITOR
+				song
+				#else
+				foo->file
+				#endif
+				);
+	if (title)
+	{
+		gtk_entry_set_text (GTK_ENTRY(tag_title), title);
+	}
+	if (artist)
+	{
+		gtk_entry_set_text (GTK_ENTRY(tag_artist), artist);
+	}
+	if (album)
+	{
+		gtk_entry_set_text (GTK_ENTRY(tag_album), album);
+	}
+	if (comment)
+	{
+		gtk_entry_set_text (GTK_ENTRY(tag_comment), comment);
+	}
+	if (genre)
+	{
+		gtk_combo_box_append_text (GTK_COMBO_BOX(tag_genre), genre);
+	}
+
 	genre_model = gtk_combo_box_get_model (GTK_COMBO_BOX(tag_genre));
 	n = gtk_tree_model_iter_n_children (genre_model, NULL);
 	gtk_combo_box_set_active (GTK_COMBO_BOX(tag_genre), n-1);
 
 	/* Audio Information */
-	sec = taglib_audioproperties_length(properties) % 60;
-    	min = (taglib_audioproperties_length(properties) - sec) / 60;
+	#ifdef HAVE_TAGEDITOR
+	guint sec = taglib_audioproperties_length(properties) % 60;
+    	guint min = (taglib_audioproperties_length(properties) - sec) / 60;
 	info = g_strdup_printf ("%02i:%02i", min, sec);
 	gtk_label_set_text (GTK_LABEL(tag_info_length), info);
+	#else
+	char *tok = NULL;
+	info = (char*) g_malloc0 (sizeof(char)*32);
+	gimmix_get_progress_status (gmo, NULL, info);
+	tok = strtok (info, "/");
+	tok = strtok (NULL, "/");
+	g_strstrip (tok);
+	gtk_label_set_text (GTK_LABEL(tag_info_length), tok);
+	#endif
 	g_free (info);
 
+	#ifdef HAVE_TAGEDITOR
 	info = g_strdup_printf ("%i Kbps", taglib_audioproperties_bitrate(properties));
+	#else
+	info = g_strdup_printf ("%i Kbps", mpd_status_get_bitrate(gmo));
+	#endif
 	gtk_label_set_text (GTK_LABEL(tag_info_bitrate), info);
 	g_free (info);
 	
+	#ifdef HAVE_TAGEDITOR
 	info = g_strdup_printf ("%i", taglib_audioproperties_channels(properties));
+	#else
+	info = g_strdup_printf ("%i", mpd_status_get_channels(gmo));
+	#endif
 	gtk_label_set_text (GTK_LABEL(tag_info_channels), info);
 	g_free (info);
 	
+	#ifdef HAVE_TAGEDITOR
 	taglib_tag_free_strings ();
+	#else
+	free (foo);
+	#endif
 	
 	return TRUE;
 }
@@ -146,14 +242,19 @@ gimmix_tag_editor_populate (const gchar *song)
 static void
 gimmix_tag_editor_close (GtkWidget *widget, gpointer data)
 {
+	#ifdef HAVE_TAGEDITOR
 	taglib_tag_free_strings ();
-    if (file)
-    	taglib_file_free (file);
-    gtk_widget_hide (tag_editor_window);
-    
-    return;
+	if (file)
+    	{
+    		taglib_file_free (file);
+    	}
+	#endif
+	gtk_widget_hide (tag_editor_window);
+
+	return;
 }
 
+#ifdef HAVE_TAGEDITOR
 static void
 gimmix_tag_editor_save (GtkWidget *button, gpointer data)
 {
@@ -199,6 +300,7 @@ gimmix_tag_editor_save (GtkWidget *button, gpointer data)
 	
 	return;
 }
+#endif
 
 void
 gimmix_tag_editor_show (void)
@@ -212,13 +314,22 @@ gimmix_tag_editor_show (void)
 	if (status == PLAY || status == PAUSE)
 	{
 		info = mpd_playlist_get_current_song (gmo);
+		#if HAVE_TAGEDITOR
 		song = g_strdup_printf ("%s/%s", cfg_get_key_value(conf, "music_directory"), info->file);
 		if (gimmix_tag_editor_populate (song))
 			gtk_widget_show (GTK_WIDGET(tag_editor_window));
+		#else
+		if (gimmix_tag_editor_populate (info))
+			gtk_widget_show (GTK_WIDGET(tag_editor_window));
+		#endif
 		else
 		{	
+			#ifdef HAVE_TAGEDITOR
 			g_warning (_("Invalid music directory."));
 			gimmix_tag_editor_error (dir_error);
+			#else
+			gimmix_tag_editor_error (_("An error occurred while trying to get song information. Please try again."));
+			#endif
 		}	
 		g_free (song);
 	}
@@ -226,6 +337,13 @@ gimmix_tag_editor_show (void)
 	return;
 }
 
+void
+gimmix_tag_editor_set_cover_image (GdkPixbuf *image)
+{
+	gtk_image_set_from_pixbuf (GTK_IMAGE(tag_editor_cover_image), image);
+	return;
+}
+
 void
 gimmix_tag_editor_error (const gchar *error_text)
 {
