@@ -1,7 +1,7 @@
 /*
  * gimmix.c
  *
- * Copyright (C) 2006-2008 Priyank Gosalia
+ * Copyright (C) 2006-2009 Priyank Gosalia
  *
  * Gimmix is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -129,10 +129,16 @@ gimmix_connect (void)
 	
 	if (mpd_connect(gmo) == MPD_OK)
 	{
-		mpd_send_password (gmo);
-		printf ("connected to mpd\n");
-		mpd_signal_connect_connection_changed (gmo, (ConnectionChangedCallback)gimmix_mpd_connection_changed_callback, NULL);
-		return true;
+		if (mpd_send_password (gmo) != MPD_OK)
+		{
+			return false;
+		}
+		else
+		{
+			printf ("connected to mpd\n");
+			mpd_signal_connect_connection_changed (gmo, (ConnectionChangedCallback)gimmix_mpd_connection_changed_callback, NULL);
+			return true;
+		}
 	}
 	else
 	{
@@ -196,6 +202,7 @@ gimmix_about_show (void)
 				"\nwejpconfig (Gimmix's configuration system)\nJohannes Heimansberg <wejpilot@gmail.com>",
 				"\nA part of the song seek code borrowed from Pygmy.",
 				"\nGimmix's Build system\nAlex Smith",
+				"\nManual page\nVincent Legout <vincent@legout.info>",
 				"\nTesting & Bug Reporting (Gimmix 0.1x)\nRohan Dhruva <rohandhruva@gmail.com>",
 				 NULL
 					};
@@ -205,10 +212,10 @@ gimmix_about_show (void)
 				"Turkish (tr) - Yavuz Selim Burgu <turkalinux@gmail.com>\n";
 
 
-	gtk_show_about_dialog (NULL,
+	gtk_show_about_dialog (GTK_WINDOW(main_window),
                            "name", APPNAME,
                            "version", VERSION,
-                           "copyright", _("\xC2\xA9 2006-2008 Priyank Gosalia  (GPL)"),
+                           "copyright", _("\xC2\xA9 2006-2009 Priyank Gosalia  (GPL)"),
                            "comments", _("Gimmix is a graphical music player daemon (MPD) client written in C."),
                            "license", license,
                            "authors", authors,
@@ -256,7 +263,7 @@ main (int argc, char *argv[])
 				vstr = g_strdup_printf ("%s %s\n%s\n",
 							g_ascii_strdown(APPNAME,strlen(APPNAME)),
 							VERSION,
-							"Copyright 2006, 2007, 2008 Priyank Gosalia");
+							"Copyright 2006, 2007, 2008, 2009 Priyank Gosalia");
 				fprintf (stdout, vstr);
 				g_free (vstr);
 				goto cleanup;
